@@ -36,10 +36,29 @@ class Bitbucket(object):
             url=url,
             auth=auth,
             data=kwargs)
-        if r.send():
-            return (r.response.status_code, r.response.text)
+        send = r.send()
+        status = r.response.status_code
+        text = r.response.text
+        error = r.response.error
+        if send:
+            if status >= 200 and  status < 300:
+                if text:
+                    try:
+                        return (True, json.loads(text))
+                    except TypeError:
+                        pass
+                    except ValueError:
+                        pass
+                return (True, text)
+            elif status >= 300 and  status < 400:
+                return (False, 'Unauthorized acces, '
+                    'please check your credentials.')
+            elif status >= 400 and  status < 500:
+                return (False, 'Service not found.')
+            elif status >= 500 and  status < 600:
+                return (False, 'Server error.')
         else:
-            return False
+            return (False, error)
 
     def url(self, action, **kwargs):
         """ Construct and return the URL for a specific API service. """
@@ -56,8 +75,8 @@ class Bitbucket(object):
             format='src')
         dir_url = url + dir
         response = self.dispatch('GET', dir_url, auth=self.auth)
-        if response and response[0] == 200:
-            repo_tree = json.loads(response[1])
+        if response[0] and isinstance(response[1], dict):
+            repo_tree = response[1]
             url = self.url(
                 'GET_ARCHIVE',
                 username=self.username,
@@ -128,8 +147,11 @@ class Bitbucket(object):
         username = username or self.username or ''
         url = self.url('GET_USER', username=username)
         response = self.dispatch('GET', url)
-        if response:
-            return json.loads(response[1])['user']
+        try:
+            return (response[0], response[1]['user'])
+        except TypeError:
+            pass
+        return response
 
     def public_repos(self, username=None):
         """ Returns all public repositories from an user.
@@ -138,53 +160,49 @@ class Bitbucket(object):
         username = username or self.username or ''
         url = self.url('GET_USER', username=username)
         response = self.dispatch('GET', url)
-        if response:
-            return json.loads(response[1])['repositories']
+        try:
+            return (response[0], response[1]['repositories'])
+        except TypeError:
+            pass
+        return response
 
     def get_repositories(self):
         """ Return own repositories."""
         url = self.url('GET_USER', username=self.username)
         response = self.dispatch('GET', url, auth=self.auth)
-        if response:
-            return json.loads(response[1])['repositories']
+        try:
+            return (response[0], response[1]['repositories'])
+        except TypeError:
+            pass
+        return response
 
     def get_repository(self, repo_slug=None):
         """ Get a single repository on Bitbucket and return it."""
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('GET_REPO', username=self.username, repo_slug=repo_slug)
-        response = self.dispatch('GET', url, auth=self.auth)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('GET', url, auth=self.auth)
 
     def get_tags(self, repo_slug=None):
         """ Get a single repository on Bitbucket and return its tags."""
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('GET_TAGS', username=self.username, repo_slug=repo_slug)
-        response = self.dispatch('GET', url, auth=self.auth)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('GET', url, auth=self.auth)
 
     def get_branches(self, repo_slug=None):
         """ Get a single repository on Bitbucket and return its branches."""
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('GET_BRANCHES', username=self.username, repo_slug=repo_slug)
-        response = self.dispatch('GET', url, auth=self.auth)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('GET', url, auth=self.auth)
 
     def create_repository(self, repo_name, scm='git', private=True):
         """ Creates a new repository on own Bitbucket account and return it."""
         url = self.url('CREATE_REPO')
-        response = self.dispatch('POST', url, auth=self.auth, name=repo_name, scm=scm, is_private=private)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('POST', url, auth=self.auth, name=repo_name, scm=scm, is_private=private)
 
     def update_repository(self, repo_slug=None, **kwargs):
         """ Updates repository on own Bitbucket account and return it."""
         url = self.url('UPDATE_REPO', username=self.username, repo_slug=repo_slug)
-        response = self.dispatch('PUT', url, self.auth, **kwargs)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('PUT', url, self.auth, **kwargs)
 
     def delete_repository(self, repo_slug=None):
         """ Delete a repository on own Bitbucket account.
@@ -192,8 +210,7 @@ class Bitbucket(object):
         """
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('DELETE_REPO', username=self.username, repo_slug=repo_slug)
-        response = self.dispatch('DELETE', url, auth=self.auth)
-        return response
+        return self.dispatch('DELETE', url, auth=self.auth)
 
     def add_service(self, service, repo_slug=None, **kwargs):
         """ Add a service (hook) to one of your repositories.
@@ -202,27 +219,21 @@ class Bitbucket(object):
         """
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('SET_SERVICE', username=self.username, repo_slug=repo_slug)
-        response = self.dispatch('POST', url, auth=self.auth, type=service, **kwargs)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('POST', url, auth=self.auth, type=service, **kwargs)
 
     def get_service(self, service_id, repo_slug=None):
         """ Get a service (hook) from one of your repositories.
         """
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('GET_SERVICE', username=self.username, repo_slug=repo_slug, service_id=service_id)
-        response = self.dispatch('GET', url, auth=self.auth)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('GET', url, auth=self.auth)
 
     def update_service(self, service_id, repo_slug=None, **kwargs):
         """ Update a service (hook) from one of your repositories.
         """
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('UPDATE_SERVICE', username=self.username, repo_slug=repo_slug, service_id=service_id)
-        response = self.dispatch('PUT', url, auth=self.auth, **kwargs)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('PUT', url, auth=self.auth, **kwargs)
 
     def delete_service(self, service_id, repo_slug=None):
         """ Delete a service (hook) from one of your repositories.
@@ -230,17 +241,14 @@ class Bitbucket(object):
         """
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('DELETE_SERVICE', username=self.username, repo_slug=repo_slug, service_id=service_id)
-        response = self.dispatch('DELETE', url, auth=self.auth)
-        return bool(response)
+        return self.dispatch('DELETE', url, auth=self.auth)
 
     def get_services(self, repo_slug=None):
         """ Get all services (hook) from one of your repositories.
         """
         repo_slug = repo_slug or self.repo_slug or ''
         url = self.url('GET_SERVICES', username=self.username, repo_slug=repo_slug)
-        response = self.dispatch('GET', url, auth=self.auth)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('GET', url, auth=self.auth)
 
     def get_archive(self, repo_slug=None, format='zip', prefix=''):
         """ Get one of your repositories and compress it as an archive.
@@ -250,46 +258,41 @@ class Bitbucket(object):
         """
         prefix = '%s'.lstrip('/') % prefix
         self.get_files_in_dir(repo_slug=repo_slug, dir='/')
-        with NamedTemporaryFile(delete=False) as archive:
-            with ZipFile(archive, 'w') as zip_archive:
-                for name, file in self.repo_tree.items():
-                    with NamedTemporaryFile() as temp_file:
-                        temp_file.write(file)
+        if self.repo_tree:
+            with NamedTemporaryFile(delete=False) as archive:
+                with ZipFile(archive, 'w') as zip_archive:
+                    for name, file in self.repo_tree.items():
+                        with NamedTemporaryFile(delete=False) as temp_file:
+                            temp_file.write(file)
                         zip_archive.write(temp_file.name, prefix + name)
-        return archive.name
+            return (True, archive.name)
+        return (False, 'Could not archive your project.')
 
     def get_ssh_keys(self):
         """ Get all ssh keys associated with your account.
         """
         url = self.url('GET_SSH_KEYS')
-        response = self.dispatch('GET', url, auth=self.auth)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('GET', url, auth=self.auth)
 
     def get_ssh_key(self, key_id=1):
         """ Get one of the ssh keys associated with your account.
         """
         url = self.url('GET_SSH_KEY', key_id=key_id)
-        response = self.dispatch('GET', url, auth=self.auth)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('GET', url, auth=self.auth)
 
     def set_ssh_key(self, key=None, label=None):
         """ Associate an ssh key with your account and return it.
         """
         key = '%s' % key
         url = self.url('SET_SSH_KEY')
-        response = self.dispatch('POST', url, auth=self.auth, key=key, label=label)
-        if response:
-            return json.loads(response[1])
+        return self.dispatch('POST', url, auth=self.auth, key=key, label=label)
 
     def delete_ssh_key(self, key_id=None):
         """ Delete one of the ssh keys associated with your account.
             Please use with caution as there is NO confimation and NO undo.
         """
         url = self.url('DELETE_SSH_KEY', key_id=key_id)
-        response = self.dispatch('DELETE', url, auth=self.auth)
-        return response
+        return self.dispatch('DELETE', url, auth=self.auth)
 
     #  ========
     #  = URLs =
